/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.visitor.impl;

import com.yiring.app.domain.visitor.Visitor;
import com.yiring.app.domain.visitor.VisitorRepository;
import com.yiring.app.param.visitor.VisitorParam;
import com.yiring.app.service.visitor.VisitorService;
import com.yiring.app.vo.visitor.VisitorVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import java.time.LocalDateTime;
import java.util.Optional;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

/**
 * @author tzl
 * 2022/4/19 14:12
 */
@Service
public class VisitorServiceImpl implements VisitorService {

    @Resource
    VisitorRepository visitorRepository;

    @Override
    public Result<String> saveVisitor(VisitorParam visitorParam) {
        if (hasVisitorMobile(visitorParam.getVisitorMobile())) {
            return Result.no(Status.BAD_REQUEST, "您输入的手机号码已经预约");
        }
        if (hasCarNum(visitorParam.getVisitorCarNum())) {
            return Result.no(Status.BAD_REQUEST, "您输入的身份证号码已经预约");
        }
        //设置创建时间和默认状态
        Visitor visitor = Visitor.builder().createTime(LocalDateTime.now()).build();
        BeanUtils.copyProperties(visitorParam, visitor);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<String> deleteVisitor(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        visitorRepository.delete(optional.get());
        return Result.ok();
    }

    @Override
    public Result<String> updateVisitor(VisitorParam visitorParam, IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        if (!visitor.getVisitorCarNum().equals(visitorParam.getVisitorCarNum())) {
            //当修改身份证时
            if (hasCarNum(visitorParam.getVisitorCarNum())) {
                return Result.no(Status.BAD_REQUEST, "您输入的身份证号码已经预约");
            }
        }
        if (!visitor.getVisitorMobile().equals(visitorParam.getVisitorMobile())) {
            //当修改手机号码时
            if (hasVisitorMobile(visitorParam.getVisitorMobile())) {
                return Result.no(Status.BAD_REQUEST, "您输入的手机号码已经预约");
            }
        }
        BeanUtils.copyProperties(visitorParam, visitor);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<VisitorVo> selectVisitor(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Visitor visitor = optional.get();
        VisitorVo vo = new VisitorVo();
        BeanUtils.copyProperties(visitor, vo);
        return Result.ok(vo);
    }

    public boolean hasCarNum(String carNum) {
        Visitor visitor = Visitor.builder().visitorCarNum(carNum).build();
        return visitorRepository.count(Example.of(visitor)) > 0;
    }

    public boolean hasVisitorMobile(String mobile) {
        Visitor visitor = Visitor.builder().visitorMobile(mobile).build();
        return visitorRepository.count(Example.of(visitor)) > 0;
    }
}
